# RandMaps release 1 (oy4) by David Bowland
# ./addons/eventscripts/randmaps/randmaps.py

# >>> To configure this addon please see randmaps.cfg <<<

"""
Makes servers play the map cycle in a random order. Requires only ES 2.0+
"""


import es
import os.path
import random


info = es.AddonInfo()
info.name = 'RandMaps'
info.version = '1'
info.url = 'http://addons.eventscripts.com/addons/view/randmaps'
info.basename = 'randmaps'
info.author = 'SuperDave'


dict_maps = {}

tuple_mapcycle_default = ('cs_assault', 'cs_compound', 'cs_havana', 'cs_italy', 'cs_militia', 'cs_office', 'de_aztec', 'de_cbble', 'de_chateau', 'de_dust', 'de_dust2', 'de_inferno', 'de_nuke', 'de_piranesi', 'de_port', 'de_prodigy', 'de_tides', 'de_train')
dict_options = {'randmaps_percentuntilreplay':[30, 'Percent of the mapcycle that must be played before a map is allowed back into the list of possible maps']}


def load():
   """
   Checks to make sure critical server variables are created by the config
   Loads the map cycle into dict_maps
   Calls es_map_start
   Ensures EventScripts is the map handler
   Creates the server commands if necessary
   """
   global dict_maps

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('randmaps') + '/randmaps.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/randmaps/randmaps.cfg')
   else:
      es.dbgmsg(0, 'RandMaps: Unable to load randmaps.cfg! Please ensure it is in the ./randmaps/ directory.')

   str_mapcycle_file = es.ServerVar('eventscripts_gamedir') + '/' + es.ServerVar('mapcyclefile')
   if os.path.isfile(str_mapcycle_file):
      file_mapcycle = open(str_mapcycle_file)
      list_mapcycle = file_mapcycle.readlines()
      file_mapcycle.close()
   else:
      es.dbgmsg(0, 'RandMaps: Unable to load %s! Using default map cycle.' % es.ServerVar('mapcyclefile'))
      list_mapcycle = tuple_mapcycle_default

   for str_map in list_mapcycle:
      dict_maps[str_map.replace('\n', '')] = 0

   es_map_start({'mapname':es.ServerVar('eventscripts_currentmap')})

   es.set('eventscripts_maphandler', 1)

   if not es.exists('command', 'nextmap'):
      es.regcmd('nextmap', 'randmaps/nextmap_cmd', 'nextmap [map]\nServer command for changing maps')
   if not es.exists('command', 'loadmapcycle'):
      es.regcmd('loadmapcycle', 'randmaps/loadmapcycle_cmd', 'loadmapcycle <map cycle file>\nWhere <map cycle file> is relative to game directory (./cstrike/<map cycle file>)\nServer command for loading a new map cycle')


def es_map_start(event_var):
   """
   Creates the map in the dictionary of maps if necessary
   Creates a list of the least played maps, omitting the current map if possible
   Selects a map out of that list to be the next map
   """
   global dict_maps

   int_lowest_plays = -1
   list_possible_maps = []
   if event_var['mapname']:
      dict_maps[event_var['mapname']] = int((int(es.ServerVar('randmaps_percentuntilreplay')) * len(dict_maps)) / 100) + 1
   elif not dict_maps:
      dict_maps['de_dust'] = 0

   for str_map in dict_maps:
      if dict_maps[str_map]:
         dict_maps[str_map] -= 1
      if dict_maps[str_map] < int_lowest_plays or int_lowest_plays < 0:
         list_possible_maps[:] = []
         list_possible_maps.append(str_map)
         int_lowest_plays = dict_maps[str_map]
      elif dict_maps[str_map] == int_lowest_plays:
         list_possible_maps.append(str_map)

   if len(list_possible_maps) > 1 and event_var['mapname'] in list_possible_maps:
      list_possible_maps.remove(event_var['mapname'])

   es.ServerVar('eventscripts_nextmapoverride').set(random.choice(list_possible_maps))


def unload():
   for str_option in dict_options:
      es.ServerVar(str_option).set(0)

   es.ServerVar('eventscripts_nextmapoverride').set('')


def nextmap_cmd():
   """
   nextmap [map]
   Changes the map
   """
   if es.getargc() > 1:
      es.ServerVar('eventscripts_nextmapoverride').set(es.getargv(1))

   if es.getplayercount():
      int_temp_userid = es.getuserid()
      es.give(int_temp_userid, 'game_end')
      es.fire(int_temp_userid, 'game_end', 'endgame')
   else:
      es.server.cmd('changelevel %s' % es.ServerVar('eventscripts_nextmapoverride'))


def loadmapcycle_cmd():
   """
   loadmapcycle <map cycle file>
   Loads a new map cycle where <map cycle file> is relative to the game directory (./cstrike/<map cycle file>)
   Deletes maps in the old cycle that are not in the new cycle
   Creates maps in the new cycle that were not in the old cycle
   Calls es_map_start to get a new next map
   """
   global dict_maps

   if es.getargc() > 1:
      str_mapcycle_file = es.ServerVar('eventscripts_gamedir') + '/' + es.getargv(1)
      if os.path.isfile(str_mapcycle_file):

         file_mapcycle = open(str_mapcycle_file)
         list_mapcycle = file_mapcycle.readlines()
         file_mapcycle.close()

         for int_count in range(len(list_mapcycle)):
            list_mapcycle[int_count] = list_mapcycle[int_count].replace('\n', '')

         dict_temp_maps = dict_maps.copy()
         for str_map in dict_temp_maps:
            if not str_map in list_mapcycle:
               del dict_maps[str_map]
            else:
               dict_maps[str_map] += 1

         for str_map in list_mapcycle:
            if not dict_maps.has_key(str_map):
               dict_maps[str_map] = 0

         es_map_start({'mapname':0})
      else:
         es.dbgmsg(0, 'RandMaps: Specified map cycle does not exist. <map cycle file> is relative to the game directory (./cstrike/<map cycle file>)')
   else:
      es.dbgmsg(0, 'Syntax: loadmapcycle <map cycle file>   -   <map cycle file> is relative to game directory (./cstrike/<map cycle file>)')